@@ -1,5 +1,6 @@
 import logging
 import random
+import os
 from functools import partial
 
 from pushsource import PushItem
@@ -67,12 +68,13 @@ class State(object):
     IN_REPOS = "IN_REPOS"
 
 
-@attr.s(frozen=True)
+@attr.s(frozen=True, slots=True)
 class UploadContext(object):
     client = attr.ib(default=None)
+    random = attr.ib(default=None)
 
 
-@attr.s(frozen=True)
+@attr.s(frozen=True, slots=True)
 class PulpPushItem(object):
     """Wraps a pushitem with additional info for Pulp push.
 
@@ -143,7 +145,10 @@ class PulpPushItem(object):
         Subclasses MAY override this to provide their own context
         (e.g. to cache a value rather than recalculating per upload).
         """
-        return UploadContext(client=pulp_client)
+        return UploadContext(
+            client=pulp_client,
+            random=random.Random(float(os.getenv("PUBTOOLS_SEED") or random.random())),
+        )
 
     @classmethod
     def items_by_type(cls, items):
@@ -401,17 +406,34 @@ class PulpPushItem(object):
         """
         # In order to get the unit into Pulp, we must first upload it into some
         # (any) repo from dest.
-        #
-        # TODO: consider picking the upload repo differently here (e.g. randomly) to
-        # make repo contention less likely.
         if repo_f is None:
-            repo_id = self.pushsource_item.dest[0]
+            # Because uploading to a repo will lock the repo (during import task),
+            # for the most possible concurrency it's best to try to uniformly
+            # distribute the repos used for upload. For example if we receive
+            # 100 items all for repos [a, b, c, d], we will get the best
+            # performance if each repo is used for roughly 25 uploads.
+            #
+            # Hence the random choice of a target repo.
+            repo_id = ctx.random.choice(self.pushsource_item.dest)
             repo_f = ctx.client.get_repository(repo_id)
 
         upload_tasks = f_flat_map(repo_f, self.upload_to_repo)
 
-        # TODO: should we check here and raise if refresh didn't find that we're present?
-        return f_flat_map(upload_tasks, lambda _: self.with_pulp_refreshed(ctx.client))
+        # Helper to verify that we've really got into at least one repo as a result
+        # of the upload.
+        def asserting_uploaded_ok(item):
+            if not item.in_pulp_repos:
+                msg = (
+                    "Fatal error: item supposedly uploaded successfully, "
+                    "but remains missing from Pulp: %s"
+                ) % item.pushsource_item
+                raise RuntimeError(msg)
+            return item
+
+        updated_f = f_flat_map(
+            upload_tasks, lambda _: self.with_pulp_refreshed(ctx.client)
+        )
+        return f_map(updated_f, asserting_uploaded_ok)
 
     def ensure_uptodate(self, client):
         """Ensure that this item is up-to-date in Pulp.
@@ -426,9 +448,21 @@ class PulpPushItem(object):
         LOG.info("Updating fields on %s", self.pushsource_item.name)
         update = client.update_content(self.unit_for_update)
 
-        # TODO: should we check after the refresh that we now consider ourselves
-        # to be up-to-date?
-        return f_flat_map(update, lambda _: self.with_pulp_refreshed(client))
+        # Helper to verify that our update really did make the item up-to-date.
+        def asserting_updated_ok(item):
+            if item.pulp_state not in (State.PARTIAL, State.IN_REPOS):
+                msg = (
+                    "Fatal error: item supposedly updated successfully, "
+                    "but actual and desired state still differ:\n"
+                    "  item:         %s\n"
+                    "  current unit: %s\n"
+                    "  desired unit: %s"
+                ) % (item.pushsource_item, item.pulp_unit, item.unit_for_update)
+                raise RuntimeError(msg)
+            return item
+
+        updated_f = f_flat_map(update, lambda _: self.with_pulp_refreshed(client))
+        return f_map(updated_f, asserting_updated_ok)
 
     @property
     def can_pre_push(self):
