@@ -23,7 +23,7 @@ class LoadPushItems(Phase):
       checksums and will definitely be missing any info regarding the Pulp state.
 
     Side-effects:
-    - none.
+    - populates items_known, items_count on the context.
     """
 
     def __init__(self, context, source_urls, **_):
@@ -31,6 +31,8 @@ class LoadPushItems(Phase):
         self._source_urls = source_urls
 
     def run(self):
+        count = 0
+
         for source_url in self._source_urls:
             with Source.get(source_url) as source:
                 LOG.info("Loading items from %s", source_url)
@@ -38,5 +40,12 @@ class LoadPushItems(Phase):
                     pulp_item = PulpPushItem.for_item(item)
                     if pulp_item:
                         self.put_output(pulp_item)
+                        count += 1
                     else:
                         LOG.info("Skipping unsupported type: %s", item)
+
+        # We know exactly how many items we're dealing with now.
+        # Set this on the context, which allows for more accurate progress
+        # info.
+        self.context.items_count = count
+        self.context.items_known.set()
